classdef mmCom_MelfaVirtSim < hgsetget
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    1     18.7.2011 14:54:38       Initial revision

  properties 
    verbose; % verbosity used to debug comunication
  end

  properties (SetAccess = private )
    
    %robot structure
    robot;
    
    %speed modifier
    spdConstant;
    
    %base speed constant
    baseSpeed;
    %jog interpolation speed constant
    jogSpeed;
    %linear interpolation speed constant
    linSpeed;
    
    %current joint coordinates
    jCoordinates;
    %current trajectory
    jTrajectory;
    %current poisition in the trajectory
    jTrajectoryPos;
    %moving in linear interpolation
    linMove;
    %whether moving
    moving;
    %time when the state was evaluated
    evalTime; 
    %variable storage
    vars;
    
    %calculated speed for the current movement in linear interpolation
    movLinSpeed;
    %calculated speed for the current movement in jog interpolation
    movJogSpeed;
    
    %communication buffer
    buffer;
    %communication timeout
    timeout;
    
    %whether received command open
    cmdOpen;
    %whether servo is on
    srvOn;
    %wheter control is on
    cntlOn;
    %wheter alarm is on
    alarmOn;
    
    %acquired options
    acquiredToRobotNo;
    acquiredToSlotNo;
    
  end

  properties (SetAccess = private, GetAccess = private )
    %whether communication is opened
    opened;  
    %figure to show the robot state
    fig;
    %control label handle
    cntlLabel;
    %servo label handle
    srvLabel;
    %alarm label handle
    alarmLabel;
    %moving label handle
    movLabel;
    %refresh timer for the figure handle
    refreshTimer;
    %handle to the Robot Toolbox robot
    rtrobot;
  end

  methods

    % ------------------------------------------------------------------------------
    function s = mmCom_MelfaVirtSim()
    %MMCOM_MELFAVIRTSIM/MMCOM_MELFAVIRTSIM  Constructor.

      s.opened=0;
      s.verbose = 0;
      s.robot=[];
      s.spdConstant=10^-1;
    
      s.baseSpeed=100;
      s.jogSpeed=100;
      s.linSpeed=10000;

      s.jCoordinates=[0;0;0;0;0;0];
      s.jTrajectory=[];
      s.jTrajectoryPos=0;
      s.linMove=0;
      s.movLinSpeed=0;
      s.movJogSpeed=[0;0;0;0;0;0];
      s.moving=0;
      s.evalTime=hat;

      s.vars={};

      s.buffer=[];
      s.timeout=5;
      
      s.cmdOpen=0;
      s.srvOn=0;
      s.cntlOn=0;
      s.alarmOn=0;
      
      s.acquiredToRobotNo=-1;
      s.acquiredToSlotNo=-1;

      s.fig=[];
      s.cntlLabel=[];
      s.srvLabel=[];
      s.alarmLabel=[];
      s.movLabel=[];
      s.refreshTimer=[];
      
      s.setVar('M_JOVRD', 100);
      s.setVar('M_OVRD',  100);
      s.setVar('M_SPD', 10000);
      
      s.setVar('M_ACL',   NaN);
      s.setVar('M_DACL',  NaN);
      
    end % fcn

    % ------------------------------------------------------------------------------
    function delete(~)
    %MMCOM_MELFAVIRTSIM/DELETE  Destructor.

    end % fcn

    % ------------------------------------------------------------------------------
    function open( s, robot )
    %MMCOM_MELFAVIRTSIM/OPEN  Open COM port.
    %
    %   s.open( robot )
    %
    %   Input:
    %     robot .. control structure, fields used:
    %       .portname
    %       .comprops

      if( s.opened )
        error( 'VSYM is already opened.' );
      end

      s.robot = robot;
      s.opened = 1;
    
      if (isstruct(robot.comprops))
        
        comprops = fieldnames(robot.comprops);
        
        for item = { comprops{:} } %iterate over the properties and set them on the serial
          set( s, item{1}, robot.comprops.(item{1}) );
        end
      end
    
      s.figCreate();
    
    end % fcn

    % ------------------------------------------------------------------------------
    function figCreate(s)
    %MMCOM_MELFAVIRTSIM/FIGCREATE Opens the figure if closed
    
      if (~isempty(s.fig))
        return;
      end
    
      s.rtrobot = mmrtCreateInteractiveRobot(s.robot);
      s.fig = figure('Name',[s.robot.description ' [Simulator]'], 'NumberTitle', 'off', 'ToolBar', 'figure', 'DeleteFcn', @s.figDelete);
      
      s.cntlLabel = uicontrol('Style','text','Units','Normalized','Position',[.60 .94 .10 .04], 'HorizontalAlignment', 'center', ...
        'String', 'CNTL', 'BackgroundColor', [1 0 0]);
      
      s.srvLabel = uicontrol('Style','text','Units','Normalized','Position',[.70 .94 .10 .04], 'HorizontalAlignment', 'center', ...
        'String', 'SERVO', 'BackgroundColor', [1 0 0]);
      
      s.movLabel = uicontrol('Style','text','Units','Normalized','Position',[.80 .94 .10 .04], 'HorizontalAlignment', 'center', ...
        'String', 'STOPPED', 'BackgroundColor', [1 0 0]);
      
      s.alarmLabel = uicontrol('Style','text','Units','Normalized','Position',[.90 .94 .10 .04], 'HorizontalAlignment', 'center', ...
        'String', 'OK', 'BackgroundColor', [0 1 0]);
            
      s.refreshTimer = timer('TimerFcn', @s.figCallback, 'Period', 0.05, ...
        'ExecutionMode', 'fixedSpacing','BusyMode','drop');
      
      plot(s.rtrobot, (s.jCoordinates')*pi/180);
      
      start(s.refreshTimer);
      
    end
    
    % ------------------------------------------------------------------------------
    function figCallback(s, ~, ~)
    %MMCOM_MELFAVIRTSIM/FIGCALLBACK Callback for the refresh timer on the figure
      
      %update the state
      s.evalState();
      
      set(s.cntlLabel, 'BackgroundColor', [~s.cntlOn s.cntlOn 0]);
      set(s.srvLabel, 'BackgroundColor', [~s.srvOn s.srvOn 0]);
      
      if (s.alarmOn)
        set(s.alarmLabel, 'BackgroundColor', [1 0 0]);
        set(s.alarmLabel, 'String', 'ERROR');
      else
        set(s.alarmLabel, 'BackgroundColor', [0 1 0]);
        set(s.alarmLabel, 'String', 'OK');
      end
      
      if (s.moving)
        set(s.movLabel, 'BackgroundColor', [0 1 0]);
        set(s.movLabel, 'String', 'MOVING');
      else
        set(s.movLabel, 'BackgroundColor', [1 0 0]);
        set(s.movLabel, 'String', 'STOPPED');
      end
      
      %robot toolbox update plot (@#***&%##*)
      r = findobj('Tag', s.rtrobot.name);
      rr = get(r, 'UserData');
      rr.q = (s.jCoordinates')*pi/180;
      set(r, 'UserData', rr);

      %clear the figure
      plot(rr, (s.jCoordinates')*pi/180);
    end
    
    % ------------------------------------------------------------------------------
    function figDelete(s, ~, ~)
    %MMCOM_MELFAVIRTSIM/FIGDELETE Cleanup when figure is closed
      
      stop(s.refreshTimer);
      delete(s.fig);
      s.fig=[];
      s.cntlLabel=[];
      s.srvLabel=[];
      s.alarmLabel=[];
      s.movLabel=[];
      s.rtrobot=[];
      s.refreshTimer=[];
      
    end
    
    % ------------------------------------------------------------------------------
    function open2(~)
    %MMCOM_MELFAVIRTSIM/OPEN2  Opens the COM port for another robot.
    %
    %   s.open2( )

      error( 'VSYM can''t be opened twice.' );
  
    end

    % ------------------------------------------------------------------------------
    function close( s )
    %MMCOM_MELFAVIRTSIM/CLOSE  Close COM port.
    %
    %   s.close

      if( ~s.opened )
         error( 'VSYM is not opened.' );
      end
      
      if (~isempty(s.fig))
        
        stop(s.refreshTimer);
        close(s.fig);
        s.fig=[];
        s.cntlLabel=[];
        s.srvLabel=[];
        s.alarmLabel=[];
        s.movLabel=[];
        s.rtrobot = [];
        s.refreshTimer=[];
      end
      
      s.robot = [];
      s.opened = 0;
  
    end % fcn

    % ------------------------------------------------------------------------------
    function r = acquire( s, robot)
    %MMCOM_MELFAVIRTSIM/ACQUIRE  Acquire this com port for a robot.
    %
    %   s.acquire( robot )
    %
    %   Prevents other robots except this one from using this comnode. Returns 0 for success.
    %   This is not intented for multithreaded use.

      if( s.verbose )
        fprintf( 'Robot %d; Slot %d is requesting to acquire the comport.\n', robot.robotNo, robot.slotNo);
      end

      if ((s.acquiredToRobotNo ~= -1) || (s.acquiredToSlotNo ~= -1))
        r = 1;
        return;
      end

      s.acquiredToRobotNo = robot.robotNo;
      s.acquiredToSlotNo = robot.slotNo;
      r = 0;
    
    end % fcn

    % ------------------------------------------------------------------------------
    function r = release(s)
    %MMCOM_MELFAVIRTSIM/RELEASE  Releases this com port for another robots.
    %
    %   s.release()
    %
    %   Releases this com port for another robots. Returns 0 for success.
    %   This is not intented for multithreaded use.

      if( s.verbose )
        fprintf( 'Received a request to release the comport.\n');
      end

      if ((s.acquiredToRobotNo == -1) || (s.acquiredToSlotNo == -1))
        r = 1;
        return;
      end

      s.acquiredToRobotNo = -1;
      s.acquiredToSlotNo = -1;
      r = 0;
    
    end % fcn

    % ------------------------------------------------------------------------------
    function [tx, r] = scanf( s )
    %MMCOM_MELFAVIRTSIM/SCANF  Read a line from COM port.
    %
    %   tx = s.scanf()
    %
    %   Reads a line from COM port controller. Line must be terminated
    %   properly
    
      if (~s.opened)
        error('COM is not opened.');
      end
      
      %emulate the delay (20 ms), SCANF + 20 ms
      
      if (isempty(s.buffer))
        pause(s.timeout);
        fprintf('VSYM: Timeout.');
      end
      
      tx = s.buffer{1};
      
      if (length(s.buffer) ~= 1)
        s.buffer = s.buffer{2:length(s.buffer)};
      else
        s.buffer = [];
      end
      
      if( s.verbose )
        fprintf( '>>%s<<\n', tx );
      end
            
      r = 0;    
      pause(0.02);
      
    end % fcn

    % ------------------------------------------------------------------------------
    function r = printf( s, tx, varargin )
    %MMCOM_MELFAVIRTSIM/PRINTF  Write a line to COM port.
    %
    %   s.printf( tx )
    %   s.printf( format, ... )
    %
    %   The line mustn't be terminated by the Terminator.

      if (~s.opened)
        error('COM is not opened.');
      end

      if( s.verbose )
        fprintf( '%s\n', sprintf( tx, varargin{:} ));
      end
      
      %emulate the delay (20 ms), PRINTF + 20 ms
      pause(0.02);
      msg=sprintf(tx, varargin{:});
      s.cmdHandler(msg);
      r=0;
      
    end % fcn

    % ------------------------------------------------------------------------------
    function settimeout(s, timeout)
    %MMCOM_MELFAVIRTSIM/SETTIMEOUT  Set COM port read timeout (in seconds).
    %
    %   s.settimeout( timeout )
    
      s.timeout = timeout;
      
    end % fcn

    % ------------------------------------------------------------------------------
    function flush(~)
    %Only for compatibility reasons
      
    end % fcn
    
    % ------------------------------------------------------------------------------
    % SIMULATOR FUNCTIONS
    % ------------------------------------------------------------------------------
    function evalState(s)
    %MMCOM_MELFAVIRTSIM/EVALSTATE  Evaluates the current robot state.
      
      %only doing anything when moving
      if (~s.moving)
        s.evalTime=hat;
        return;
      end
      
      %evaluation time
      newTime=hat;
      haveTime=newTime-s.evalTime;
          
      while (1)
      
        %check the time needed between another trajectory point
        if (s.linMove)
          loc1=mmDkt(s.robot, s.jCoordinates);
          loc2=mmDkt(s.robot, s.jTrajectory(:,s.jTrajectoryPos+1));
          dist = sqrt(sum((loc1(1:3)-loc2(1:3)).^2));
          diffMovTime = dist/s.movLinSpeed;
        else
          %find the jog with the highest speed and use it to measure
          deltaJogs=s.jCoordinates-s.jTrajectory(:,s.jTrajectoryPos+1);
          j=find(abs(s.movJogSpeed)==max(abs(s.movJogSpeed)));
          diffMovTime=deltaJogs(j(1))/s.movJogSpeed(j(1));
        end
        
        %check if I have that time
        if (haveTime > diffMovTime)
        
          s.jTrajectoryPos=s.jTrajectoryPos+1;
          s.jCoordinates=s.jTrajectory(:,s.jTrajectoryPos);
          
          if (size(s.jTrajectory,2) == s.jTrajectoryPos)            
            %finish
            s.jTrajectoryPos=0;
            s.jTrajectory=[];
            s.moving=0;
            s.movLinSpeed=0;
            s.movJogSpeed=0;
            s.linMove=0;
            s.evalTime=newTime;
            break;
          else          
            haveTime = haveTime-diffMovTime;
            s.evalTime = s.evalTime+diffMovTime;            
          end
        
        else
          break;
        end
        
      end   
      
    end
    
    % ------------------------------------------------------------------------------
    function r = cmdHandler(s, text)
    %MMCOM_MELFAVIRTSIM/CMDHANDLER  Handles the command sent via serial line.
    % Return value r:
    %   0 - GOOD
    %   1 - ERROR
    %   2 - INVALID ROBOT
        
      %evaluate state 
      s.evalState();
      
      %validate
      robotSgn = sprintf('%d;%d;', s.robot.robotNo, s.robot.slotNo);
      sgnTest = regexp(text, robotSgn);
      
      if ((~isempty(sgnTest)) && (sgnTest(1) ~= 1))
        r = 2;
        return;
      end
      
      %figure out the command
      cmd=substr(text, length(robotSgn));
      cmdname = '';
      c=1;
      while (c <= length(cmd))
        if ((cmd(c) >= 'A' && cmd(c) <= 'Z') || (cmd(c) >= '0' && cmd(c) <= '9'))
          cmdname = [cmdname cmd(c)];
          c = c + 1;  
          continue
        end
        
        break;
              
      end
      
      %check if there is equals sign after the cmd without spaces
      cmdargs=substr(cmd, c-1);
      cmdnsargs=cmdargs(cmdargs ~= ' ');
      
      %check for assignment
      if (~isempty(cmdnsargs) && cmdnsargs(1) == '=')
        
        value = substr(cmdnsargs, 1);   
        
        %first check variables not requiring control on
        switch(cmdname)
          case 'OPEN'
            %ignore te argument in here
            if (s.cmdOpen)
              r=1;
              s.pushResponse('ERROR');
            else                    
              s.cmdOpen=1;
              r=0;
              s.pushResponse();
            end
            
            return
        end
        
        %check if have control
        if (s.ensureCntlOn())
          r = 1;
          return;
        end
        
        execTest = regexp(cmdname, 'EXEC');
        
        if ((~isempty(execTest)) && (execTest(1) == 1))
          
          execCmdname = substr(cmdname, 4);
          
          %definitely not valid if the length is lesser
          if (length(execCmdname) >= 2)
                      
            %determine value based on the variable
            if ((execCmdname(1)=='P') || (execCmdname(1)=='J'))
              %assign to Px or Jx variable
              vnum=str2double(substr(execCmdname,1));
              if (~isnan(vnum))

                mflVct = s.parseMelfaVector(value);

                if (isempty(mflVct))
                  r = 1;
                  s.pushResponse('ERROR');
                  return;
                end

                s.setVar(execCmdname,mflVct);
                r=0;
                s.pushResponse();
                return
              end
            else
               if (execCmdname(1)=='M' && length(cmdnsargs) >= 2 && cmdnsargs(2)=='M') 
                %assign to Mx variable from another M variable
                vnum=str2double(substr(execCmdname,1));
                if (~isnan(vnum))
                  
                  tval = s.getVar(cmdnsargs(2:length(cmdnsargs)));
                  if (isempty(tval))
                    r = 1;
                    s.pushResponse('ERROR');
                  else
                    s.setVar(execCmdname, tval);
                    r=0;
                    s.pushResponse();           
                  end
                  
                  return
                end
               end
            end

            %check if assigned via VAL
            valassign=regexp(execCmdname,'VAL=');

            if((~isempty(valassign)) && valassign(1)==1)

              valassignString=substr(execCmdname,4);
              equalsSign=regexp(valassignString,'=', 'once');

              if (isempty(equalsSign))
                r = 1;
                s.pushResponse('ERROR');
                return;
              end

              valassignVarname = valassignString(1:equalsSign(1)-1);
              valassignValue = substr(valassignString,equalsSign(1)+1);

              %just a plain number
              res = str2double(valassignValue);

              if (isnan(res))
                r = 1;
                s.pushResponse('ERROR');
                return;
              end

              s.setVar(valassignVarname,res);
              r=0;
              s.pushResponse();
              return

            end
          end       
        end 
      end
      
      %first check the commands doesn't require control on
      switch (cmdname)
        % -----------------
        case 'CNTLON'
          if (s.cmdOpen && (~s.cntlOn) )
            s.cntlOn=1;
            r=0;
            s.pushResponse();
            return;
          end
          
          r=1;
          s.pushResponse('ERROR');
          return;
        % ----------------- 
        case 'CNTLOFF'
          if (s.cmdOpen && s.cntlOn )
            s.cntlOn=0;
            r=0;
            s.pushResponse();
            return;
          end
          
          r=1;
          s.pushResponse('ERROR');
          return;
        % ----------------- 
        case 'CLOSE'
          if (s.cmdOpen && (~s.cntlOn) )
            s.cmdOpen=0;
            r=0;
            s.pushResponse();
            return;
          end
          
          r=1;
          s.pushResponse('ERROR');
          return;
        % ----------------- 
        case 'OPEN'
          if (~s.cmdOpen)
            s.cmdOpen=1;
            r=0;
            s.pushResponse();
            return;
          end
          
          r=1;
          s.pushResponse('ERROR');
          return;
        % ----------------- 
        case 'RSTALRM'
          r=0;
          s.alarmOn=0;
          s.pushResponse();
          return;
        % -----------------
        case 'DSTATE'     
          runst=0;
          
          if (s.cntlOn)
            runst=runst+128;
          end
          
          if (s.srvOn)
            runst=runst+32;
          end
          
          if (s.moving)
            runst=runst+64;
          end
          
          s.pushResponse(sprintf('%s00%d;0;0',dec2hex(runst),s.alarmOn));
          r=0;
          return;
      end
      
      %the following commands require cntlOn
      if (s.ensureCntlOn())
        r = 1;
        return;
      end
      
      switch(cmdname)
        % -----------------
        case 'EXECMOV'
          
          if (s.ensureReadyToMove())
            r = 1;
            return;
          end
          
          if (isempty(cmdnsargs))
            s.pushResponse('ERROR');
            r = 1;
            return;
          end
          
          vct=s.getVar(cmdnsargs);
          
          if (isempty(vct))
            s.pushResponse('CRITICAL_ERROR');
            r = 1;
            return;
          end
        
          %get the trajectory
          coordinates=vct.coordinates;
                    
          s.jTrajectory = mmSimulateMove(s.robot, 'J', s.jCoordinates, cmdnsargs(1), coordinates);
          
          %empty trajectory - error
          if (isempty(s.jTrajectory))
            s.pushResponse('CRITICAL_ERROR');
            r = 1;
            return;
          end 
          
          %one field in trajectory - idle
          if (size(s.jTrajectory,2)==1);
            s.jTrajectory=[];
            s.pushResponse();
            r = 0;
            return;
          end
          
          %calculate the jogSpeed
          
          %get the delta
          deltaJogs=s.jTrajectory(:,1)-s.jTrajectory(:,size(s.jTrajectory,2));
          %normalize it
          deltaJogs=deltaJogs/max(abs(deltaJogs));
          %multiply it by speed
          s.movJogSpeed=deltaJogs*s.baseSpeed*s.jogSpeed*s.spdConstant*0.36;
                  
          s.jTrajectoryPos=1;
          s.linMove=0;
          s.moving=1;
                  
          s.pushResponse();
          r = 0;
          return;
          
        % -----------------
        case 'EXECMVS'
          
          if (s.ensureReadyToMove())
            r = 1;
            return;
          end
          
          if (isempty(cmdnsargs))
            s.pushResponse('ERROR');
            r = 1;
            return;
          end
          
          vct=s.getVar(cmdnsargs);
          
          if (isempty(vct))
            s.pushResponse('CRITICAL_ERROR');
            r = 1;
            return;
          end
          
          coordinates=vct.coordinates;
          
          %get the trajectory
          s.jTrajectory = mmSimulateMove(s.robot, 'P', s.jCoordinates, cmdnsargs(1), coordinates);
          
          %empty trajectory - error
          if (isempty(s.jTrajectory))
            s.pushResponse('CRITICAL_ERROR');
            r = 1;
            return;
          end 
          
          %one field in trajectory - idle
          if (size(s.jTrajectory,2)==1);
            s.jTrajectory=[];
            s.pushResponse();
            r = 0;
            return;
          end
          
          s.jTrajectoryPos=1;
          s.linMove=1;
          s.movLinSpeed=s.baseSpeed*s.linSpeed*s.spdConstant/100;
          s.moving=1;
          
          s.pushResponse();
          r = 0;
          
        % -----------------
        case 'EXECJOVRD'
          
          if (s.ensureNotMoving())
            r = 1;
            return;
          end
          
          num=str2double(cmdnsargs);
          if (~isnan(num))
            s.jogSpeed=num;
            s.setVar('M_JOVRD',num);
            s.pushResponse();
            r = 0;
          else
            s.pushResponse('ERROR');
            r = 1;
          end
        % ----------------- 
        case 'EXECOVRD'
          
          if (s.ensureNotMoving())
            r = 1;
            return;
          end
          
          num=str2double(cmdnsargs);
          if (~isnan(num))
            s.baseSpeed=num;
            s.setVar('M_OVRD',num);
            s.pushResponse();
            r = 0;
          else
            s.pushResponse('ERROR');
            r = 1;
          end
        % ----------------- 
        case 'EXECSPD'
          
          if (s.ensureNotMoving())
            r = 1;
            return;
          end
          
          num=str2double(cmdnsargs);
          if (~isnan(num))
            s.linSpeed=num;
            s.setVar('M_SPD',num);
            s.pushResponse();
            r = 0;
          else
            s.pushResponse('ERROR');
            r = 1;
          end
        % -----------------         
        case 'EXECACCEL'      
          %ignore
          s.pushResponse();
          r = 0;
          return
        % ----------------- 
        case 'EXECBASE'
          r=0;
          s.pushResponse();
          return;
        % ----------------- 
        case 'EXECTOOL'
          r=0;
          s.pushResponse();
          return;
        % ----------------- 
        case 'SRVON'      
          if  (~s.srvOn)
            s.srvOn=1;
            s.pushResponse();
            r = 0;
          else
            s.pushResponse('ERROR');
            r = 1;
          end
          
          return;
        % ----------------- 
        case 'SRVOFF'       
          if (s.ensureNotMoving())
            r = 1;
            return;
          end
          
          if  (s.srvOn)
            s.srvOn=0;
            s.pushResponse();
            r = 0;
          else
            s.pushResponse('ERROR');
            r = 1;
          end
          
          return;
        % ----------------- 
        case 'STOP'
          s.jTrajectory=[];
          s.jTrajectoryPos=0;
          s.moving=0;
          s.movLinSpeed=0;
          s.movJogSpeed=0;
          s.linMove=0;
          r=0;
          s.pushResponse();
          return;
        % -----------------
        case 'JPOSF'      
          if (~s.moving)
            movSpeed=0;
          elseif (s.linMove)
            movSpeed=s.baseSpeed*s.linSpeed*s.spdConstant/100;
          else
            movSpeed=s.baseSpeed*s.jogSpeed*s.spdConstant;
          end
          
          s.pushResponse(sprintf('J1;%.2f;J2;%.2f;J3;%.2f;J4;%.2f;J5;%.2f;J6;%.2f;;****,****;%d;%.2f;00000000',...
                        s.jCoordinates',s.baseSpeed,movSpeed));
                      
          
          r=0;
          return;
        % -----------------
        case 'PPOSF'          
          if (~s.moving)
            movSpeed=0;
          elseif (s.linMove)
            movSpeed=s.baseSpeed*s.linSpeed*s.spdConstant/100;
          else
            movSpeed=s.baseSpeed*s.jogSpeed*s.spdConstant;
          end
          
          P=mmDkt(s.robot, s.jCoordinates);
          F=mmGetFlags(s.robot, s.jCoordinates);
          s.pushResponse(sprintf('X;%.2f;Y;%.2f;Z;%.2f;A;%.2f;B;%.2f;C;%.2f;;%d,%s;%d;%.2f;00000000',...
                        P',F(1),dec2hex(F(2),8),s.baseSpeed,movSpeed));

          r=0;
          return;
          
        % -----------------
        case 'VAL'
          if (length(cmdnsargs) >= 2 && cmdnsargs(1)==s.robot.topVariableSign && cmdnsargs(2) == ';')       
            val=s.getVar('M1');
            
            if (isempty(val))
              val=0;
            end
            
            r=0;
            s.pushResponse(sprintf('M1=%.2f',val));
          else
            r=1;
            s.pushResponse('ERROR');
          end
          
          return;
        % ----------------- 
        otherwise
          r=1;
          s.pushResponse('ERROR');
          return; 
      end
      
    end
    
    % ------------------------------------------------------------------------------
    function r = ensureCntlOn(s)
    %MMCOM_MELFAVIRTSIM/ENSURECNTLON  Ensures if the control is on; fires error if not
      
      if (~(s.cntlOn) || s.alarmOn)
        s.pushResponse('ERROR');
        r = 1;
      else
        r = 0;
      end
      
    end
    
    % ------------------------------------------------------------------------------
    function r = ensureReadyToMove(s)
    %MMCOM_MELFAVIRTSIM/ENSUREREADYTOMOVE  Ensures if can move now; fires error if not
      
      if (ensureCntlOn(s))
        r = 2;
      elseif (s.moving)
        s.pushResponse('ERROR');
        r = 3;
      elseif (~(s.srvOn))
        s.pushResponse('CRITICAL_ERROR');
        r = 1;
      else
        r = 0;
      end
      
    end
    
    % ------------------------------------------------------------------------------
    function r = ensureNotMoving(s)
    %MMCOM_MELFAVIRTSIM/ENSURENOTMOVING  Ensures if not moving; fires error if moving
      
      if (s.moving)
        s.pushResponse('ERROR');
        r = 1;
      else
        r = 0;
      end
      
    end
    
    % ------------------------------------------------------------------------------
    function pushResponse(s,resp)
    %MMCOM_MELFAVIRTSIM/PUSHRESPONSE  Pushes response; recoginses ERROR and CRITICAL_ERROR for errors
      
      if (nargin > 1)
        if (strcmp(resp, 'ERROR'))
          newMsg = sprintf('QError');
        elseif (strcmp(resp, 'CRITICAL_ERROR'))
          newMsg = sprintf('QError');
          s.raiseAlarm();
        else
          newMsg = sprintf('QoK%s', resp);
        end
      else
        newMsg = sprintf('QoK');
      end
      
      s.buffer = [ s.buffer; {newMsg}];
    end
    
    % ------------------------------------------------------------------------------
    function raiseAlarm(s)
    %MMCOM_MELFAVIRTSIM/RAISEALARM  Raises the alarm, turns servo off and stops movement
      
      if (~s.alarmOn)
        s.alarmOn=1;
        
        %stop & servo OFF
        s.jTrajectory=[];
        s.jTrajectoryPos=0;
        s.moving=0;
        s.movLinSpeed=0;
        s.movJogSpeed=0;
        s.linMove=0;
        s.srvOn=0;
      end
    end
    
    % ------------------------------------------------------------------------------
    function setVar(s,var,val)
    %MMCOM_MELFAVIRTSIM/SETVAR  Sets a variable.
      
      found=0;
      
      for i=1:size(s.vars,1)
        if (strcmp(s.vars{i,1},var))
          s.vars{i,2}=val;
          found=1;
          break;
        end
      end
      
      if (~found)
        s.vars = [ s.vars; {var, val} ];
      end
      
    end
    
    % ------------------------------------------------------------------------------
    function val = getVar(s,var)
    %MMCOM_MELFAVIRTSIM/GETVAR  Gets a variable.
      
      for i=1:size(s.vars,1)
        if (strcmp(s.vars{i,1},var))
          val=s.vars{i,2};
          return
        end
      end
      
      val=[];
    end
      
    % ------------------------------------------------------------------------------
    function mflVct = parseMelfaVector(~,cmd)
    %MMCOM_MELFAVIRTSIM/PARSEMELFAVECTOR  Parses a vector send via link (jog or linear coordinates) 
      
      %check for coordinates
      p = 2;
      
      if (cmd(1) ~= '(')
        mflVct = [];
        return;
      end
      
      coords = [];
      commas = regexp(cmd, ',');
      
      i = 1;
      
      while (1)
      
        if (i > length(commas))
          endindex=regexp(cmd, ')');
          endindex=endindex(1);
          last=1;
        else
          endindex=commas(i);
          last=0;
        end
        
        res = str2double(substr(cmd, p - 1, endindex - p));
        
        if (isnan(res))
          mflVct = [];
          return;
        end
        
        coords = [coords; res];
        p = endindex + 1;
        
        if (last)
          break;
        end
        i=i+1;
      end
      
      cmd = substr(cmd,p-1);
      mflVct.coordinates = coords;
      mflVct.flags = [];
      mflVct.args = '';
      
      if (~isempty(cmd))
        %check for flags (fixed length of two)
        if (cmd(1) == '(')
          %flags
          comma = regexp(cmd, ',');
          semicolon = comma(1);
          
          roundbracket = regexp(cmd, ')');
          roundbracket = roundbracket(1);
          
          f1 = str2double(substr(cmd, 1, comma - 2));
          f2 = str2double(substr(cmd, semicolon + 1, roundbracket - comma - 1));
          
          mflVct.flags = [f1;f2];
          
          cmd = substr(cmd, roundbracket);
          
        end
        
        %check for arguments
        %check again for emptyness (in case if there be flags but no arguments)
        if (~ismpty(cmd))
          mflVct.args=cmd;
        end 
        
      end
      
    end
    
  % ------------------------------------------------------------------------------

  end % meths

end % cls
