classdef mmCom_Serial_Procedural < handle
%MMCOM_SERIAL_PROCEDURAL Interface to low-level COM port IO via Matlab's serial object.
%   Use when communicating with the robot in procedural mode. Adapted from BlueBot toolbox.
%
%   Properties:
%     timeout         .. (ro) Last used timeout.
%     verbose         .. (rw) Verbosity, nozero is used to debug communication.
%
%   Methods:
%     open            .. Open COM port.
%     open2           .. Open COM port for another robot
%     close           .. Close COM port.
%     acquire         .. Acquires the COM port for robot.
%     release         .. Releases the COM port for another robots.
%     scanf           .. Reads from COM port.
%     printf          .. Writes to COM port.
%     settimeout      .. Set COM port read timeout.
%     flush           .. Flush reading and writing on COM port.
%
%   Static methods:
%     reset           .. Try to close and free all COM ports in use.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Pavel Krsek, Martin Matousek, Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    2     25.5.2010 15:28:05       MM adaptation, procedural protocol driver
% Pavel Krsek                      1     17.2.2010 17:51:32       BlueBot release
% Martin Matousek                                                 

  properties 
    verbose; % verbosity used to debug comunication
  end

  properties (SetAccess = private )
    timeout; % last used timeout
    acquiredToRobotNo;
    acquiredToSlotNo;
    noOwners;
  end

  properties (SetAccess = private, GetAccess = private )
    handle;  % COM port handle (Matlab's serial port object)
    buffer;
  end

  methods( Static )

    % ------------------------------------------------------------------------------
    function reset()
    %MMCOM_SERIAL_PROCEDURAL.RESET  Try to close and free all COM ports in use.
    %  
    %   mmCom_Serial.reset

      slist = instrfind( 'type', 'serial' );
      for i = 1:length(slist)
          s = slist(i);
          if( isobject( s ) )
              if( isequal( s.status, 'open' ) )
                  fprintf( 'Closing ''%s''\n', s.Port );
                  fclose( s );
              end
              fprintf( 'Deleting serial object for ''%s''.\n', s.Port );
              delete( s );
          end
      end

    end % fcn
  
  end % meths

  methods

    % ------------------------------------------------------------------------------
    function s = mmCom_Serial_Procedural()
    %MMCOM_SERIAL_PROCEDURAL/MMCOM_SERIAL_PROCEDURAL  Constructor.

      s.verbose = 0;
      s.acquiredToRobotNo = -1;
      s.acquiredToSlotNo = -1;
      s.buffer = '';

    end % fcn

    % ------------------------------------------------------------------------------
    function delete(s)
    %MMCOM_SERIAL_PROCEDURAL/DELETE  Destructor.

      if( ~isempty( s.handle ) )
        fprintf( 'Closing %s port in destructor.\n', s.handle.Port );
        s.close;
      end

    end % fcn

    % ------------------------------------------------------------------------------
    function open( s, robot )
    %MMCOM_SERIAL_PROCEDURAL.OPEN  Open COM port.
    %
    %   s.open( robot )
    %
    %   Input:
    %     robot .. control structure, fields used:
    %       .portname
    %       .comprops

      if( ~isempty( s.handle ) )
        error( 'COM port is already open.' );
      end

      s.handle = serial( robot.portname );
      s.timeout = [];

      comprops = fieldnames(robot.comprops);
      for item = { comprops{:} } %iterate over the properties and set them on the serial
        set( s.handle, item{1}, robot.comprops.(item{1}) );
      end;

      fopen( s.handle );  % open serial port
      if( strcmp( s.handle.Status, 'closed' ) ) % TODO udelat positivni test
        error( 'The specified port could not be opened.' ); 
      end

      s.noOwners = 1;
  
    end % fcn


    % ------------------------------------------------------------------------------
    function open2( s )
    %MMCOM_SERIAL_PROCEDURAL.OPEN2  Opens the COM port for another robot.
    %
    %   s.open2( )

      if( ~isempty( s.handle ) )
        s.noOwners = s.noOwners + 1;
        return;
      end

      error( 'COM port is not opened, use the normal open function.' );
  
    end

    % ------------------------------------------------------------------------------
    function close( s )
    %MMCOM_SERIAL_PROCEDURAL/CLOSE  Close COM port.
    %
    %   s.close

      if( isempty( s.handle ) )
         error( 'COM port is not open.' );
      end

      if( s.noOwners > 1)
        s.noOwners = s.noOwners - 1;
        return;
      end

      fclose( s.handle );
      delete( s.handle );
      s.handle = [];
      s.noOwners = 0;
  
    end % fcn

    % ------------------------------------------------------------------------------
    function r = acquire( s, robot)
    %MMCOM_SERIAL_PROCEDURAL/ACQUIRE  Acquire this com port for a robot.
    %
    %   s.acquire( robot )
    %
    %   Prevents other robots except this one from using this comnode. Returns 0 for success.
    %   This is not intented for multithreaded use.

      if( s.verbose )
        fprintf( 'Robot %d; Slot %d is requesting to acquire the comport.\n', robot.robotNo, robot.slotNo);
      end

      if ((s.acquiredToRobotNo ~= -1) || (s.acquiredToSlotNo ~= -1))
        r = 1;
        return;
      end

      s.acquiredToRobotNo = robot.robotNo;
      s.acquiredToSlotNo = robot.slotNo;
      r = 0;
    
    end % fcn

    % ------------------------------------------------------------------------------
    function r = release(s)
    %MMCOM_SERIAL_PROCEDURAL/RELEASE  Releases this com port for another robots.
    %
    %   s.release()
    %
    %   Releases this com port for another robots. Returns 0 for success.
    %   This is not intented for multithreaded use.

      if( s.verbose )
        fprintf( 'Received a request to release the comport.\n');
      end

      if ((s.acquiredToRobotNo == -1) || (s.acquiredToSlotNo == -1))
        r = 1;
        return;
      end

      s.acquiredToRobotNo = -1;
      s.acquiredToSlotNo = -1;
      r = 0;
    
    end % fcn

    % ------------------------------------------------------------------------------
    function [tx, r] = scanf( s )
    %MMCOM_SERIAL_PROCEDURAL/READLINE  Read a line from COM port.
    %
    %   tx = s.readline
    %
    %   Reads a line from COM port controller. Line must be terminated
    %   properly

      if (isempty(s.buffer))
        r = 1;
        tx = '';
      else
        tx = s.buffer;
        s.buffer = '';
        r = 0;
      end
    end % fcn

    % ------------------------------------------------------------------------------
    function r = printf( s, tx, varargin )
    %MMCOM_SERIAL_PROCEDURAL/WRITELINE  Write a line to COM port.
    %
    %   s.printf( tx )
    %   s.printf( format, ... )
    %
    %   The line mustn't be terminated by the Terminator.
    %   Returns r = 0 if success, 100 for full buffer, 1 for NAK returned, 2 timeout, anything else for internal error 

      if( s.verbose )
        fprintf( '%s\n', sprintf( tx, varargin{:} ));
      end

      if (~isempty(s.buffer))
        r = 100;
        return;
      end
      
      %print ENQ
      fwrite(s.handle, 05);

      %check for ACK
      %give it 200 ms for reply
      tout = 200;
      t0=clock;
      
      while(1)      
        if (s.handle.BytesAvailable > 0)
          b = fread(s.handle, s.handle.BytesAvailable);
          
          %check ACK - if ACK then OK, NAK for fail
          if (b(1) ~= 06)
            r = 1;
            return;
          else
            break;
          end
        end
        
        if (etime(clock,t0) > tout)
          r = 2;
          return;
        end
      end
      
      %generate the message 
      cmdBase = sprintf(tx, varargin{:});
      cmdLen = dec2hex(length(cmdBase) + 1, 2); %+1 because of the fixed R
      cmd = ['D0' cmdLen 'R' cmdBase];
      cmpltLen = length(cmd);
      chksum = uint8(0);
      
      %checksum
      for i=1:cmpltLen
        chksum = bitxor(chksum, uint8(cmd(i)));
      end
      
      chksum = dec2hex(chksum, 2);
      
      %STX - cmd - ETX
      fwrite(s.handle, [02 cmd chksum 03]);
      
      %read reply
      tout = s.timeout;
      while(1)      
        if (s.handle.BytesAvailable > 0)
          b = fread(s.handle, s.handle.BytesAvailable);
          
          %add them to the buffer
          s.buffer = [s.buffer b'];
          
          %check for ETX
          term = strfind(s.buffer, 03);
        
          if (~isempty(term))
            repLen = term(1);
            
            %verify the checksum
            chksum = uint8(0);
            for i=2:repLen-3
              chksum = bitxor(chksum, uint8(s.buffer(i)));
            end
            
            supposedChksum = hex2dec(substr(s.buffer, repLen - 3, 2));
            
            if (chksum ~= supposedChksum)
              r = 4;
            else
              r = 0;
            end
            
            %check the length
            len = repLen - 8;
            supposedLen = hex2dec(substr(s.buffer, 3, 2));
            
            if ((len ~= supposedLen) && (r ~= 4))
              r = 3;
            end
            
            s.buffer = substr(s.buffer, 6, repLen - 9); 
            
            %write EOT
            fwrite(s.handle, 04);
            
            break;
          end
        end
        
        if (etime(clock,t0) > tout)
          r = 2;
          return;
        end
      end
            
    end % fcn

    % ------------------------------------------------------------------------------
    function settimeout( s, timeout )
    %MMCOM_SERIAL_PROCEDURAL/SETTIMEOUT  Set COM port read timeout (in seconds).
    %
    %   s.settimeout( timeout )

        set( s.handle, 'Timeout', timeout ); 
        s.timeout = timeout;

    end % fcn

    % ------------------------------------------------------------------------------
    function flush( s )
    %MMCOM_SERIAL/FLUSH  Flush reading and writing on COM port.
    %
    %   s.flush
    %
    %   Flush any pending write operation on COM port and discard any characters 
    %   from a receive buffer.

    % no write flush code

      if( s.verbose )
        fprintf( 'Flushing ...\n' );
      end

      while( s.handle.BytesAvailable )
        tx = fscanf( s.handle );
        if( s.verbose )
            fprintf( '[[%s]]\n', tx );
        end
      end

      if( s.verbose )
        fprintf( 'Flushed.\n' );
      end

    end % fcn

  % ------------------------------------------------------------------------------

  end % meths

end % cls


