function [names values numJoints r] = mmGetPos(robot, type)
%MMGETPOS Returns the current position in selected coordinates.
%COMMONARGS r, robot
%
% Usage:
% [names values numJoints r] = mmGetPos(robot, type);
%
% Get the coordinates:
% [~, values, numJoints, r] = mmGetPos(robot, type);
% coordinates = values(1:numJoints); - get the current coordinates
%
% Arguments:
%   type: the type of the coordinates (this can be 'P' for XYZ, 'J' for
%         joint and 'X' for XYZ coordinates and the rotation in joint coordinates)
% 
% Return values:
%   names:  Returns the name of the value with same index (column cell array)
%   values: The value (column vector)
%   numJoints: The values(1:numJoints) are axis coordinates
%
% The Mitsubishi Melfa IV robot returns Axis [Name, Value], Flags1, Flags2,
% Override and Speed when asked upon current position in string format. This
% string is the parsed into two arrays names and values. XYZ
% coordinates are in milimetres, joint coordinates are in degradians.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    1     30.4.2010 17:05:56       Initial revision

[reply s] = mmSendCmd(robot, '%cPOSF', type);

if (~s)
    %parse the return argument
    %first axis is at position 4
    start = 3;
    scindex = 1;
    scolons = findstr(reply, ';');
    
    if (isempty(scolons) || (size(scolons, 2) < 3))
      r = 1;
      names = {};
      numJoints = -1;
      values = [NaN; NaN; NaN; NaN; NaN; NaN];
      return;     
    end
    
    index = 1;

    names = cell(length(scolons) / 2 + 1, 1);
    values = zeros(length(scolons) / 2 + 1, 1);
    
    %parse the axes properties (ended by ';;');
    while ((start + 1) ~= scolons(scindex)) 
      names{index} = substr(reply, start, scolons(scindex) - start - 1);
      start = scolons(scindex);
      scindex = scindex + 1;
      values(index) = str2double(substr(reply, start, scolons(scindex) - start - 1));
      start = scolons(scindex);
      scindex = scindex + 1;
      index = index + 1;
    end

    if (nargout > 2)
      numJoints = index - 1;
    end

    %flags, override, end speed
    start = scolons(scindex);
    scindex = scindex + 1;

    names{index} = 'Flags1';
    flagsStr = substr(reply, start, scolons(scindex) - start - 1);
    commaPos = strfind(flagsStr, ',');
    values(index) = str2double(substr(flagsStr, 0, commaPos - 1));
    index = index + 1;
    names{index} = 'Flags2';
    values(index) = str2double(substr(flagsStr, commaPos));

    start = scolons(scindex);
    scindex = scindex + 1;
    index = index + 1;

    names{index} = 'Override';
    values(index) = str2double(substr(reply, start, scolons(scindex) - start - 1));
    start = scolons(scindex);
    scindex = scindex + 1;
    index = index + 1;

    names{index} = 'EndSpeed';
    values(index) = str2double(substr(reply, start, scolons(scindex) - start - 1));
    r = 0;
else
    names = {};
    numJoints = -1;
    values = [];
    r = s;
end
  
if (nargout > 4)
    error('Too much output arguments');
end 
  
end
