function J = mmIkt_RV6S_Series(robot, P)
%MMIKT_RV6S_SERIES Inverse kinematic task for robots for RV-6S / RV-6SDL.
%See the manual for mmIkt.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    2     12.10.2010 16:10:32      Corrected DH notation
% Martin Meloun                    1     25.5.2010 15:28:05       Initial revision

%convert to radians
P(4:6) = P(4:6)*pi/180; 
myeps = 10000*eps;

%obtain W to solve the kinematics for the first three joints
T = robot.base \ (((mtxTranslate(P(1:3)) * mtxRotateZ(P(6)) * mtxRotateY(P(5)) * mtxRotateX(P(4)))...
    / robot.tool) / robot.A76);

W = T * [0; 0; 0; 1];

%solve the ikt for first three joints
theta1a = atan2(W(2), W(1));

%possible alternate solution
theta1b = theta1a + pi;
if (theta1b > pi); theta1b = theta1b - 2*pi; end

%two circles, with c1, c2 as centers with r1 and r2 as radii
r1 = robot.denavitHartenberg(2,2);
r2 = sqrt(robot.denavitHartenberg(4,4)^2 + robot.denavitHartenberg(2,3)^2);

c1 = [robot.denavitHartenberg(2,1); 0];
absW = sqrt(sum(W(1:2).^2));
c2a = [ absW; W(3) - robot.denavitHartenberg(4,1)];
c2b = [-absW; W(3) - robot.denavitHartenberg(4,1)];

%solutions for the circle intersection 
s = []; %structure [x; y; theta1; c2]

%check for number of solutions for each of the case
%the arm can only straighten up

%case a
a = sqrt(sum((c2a-c1).^2));
xa = (r1^2+a^2-r2^2)/(2*a);
sqr_ya = r1^2 - xa^2;
if (abs(sqr_ya) < myeps) %one solution for case a
  phi = atan2(c2a(2) - c1(2), c2a(1) - c1(1));
  s = [s, [c1(1) + cos(phi)*xa; c1(2) + sin(phi)*xa; theta1a; c2a] ];
elseif (sqr_ya > 0) %two solutions for case a
  ya = sqrt(sqr_ya);
  phi = atan2(c2a(2) - c1(2), c2a(1) - c1(1));
  s = [s, [c1(1) + cos(phi)*xa - sin(phi)*ya;    c1(2) + sin(phi)*xa + cos(phi)*ya;    theta1a; c2a], ...
          [c1(1) + cos(phi)*xa - sin(phi)*(-ya); c1(2) + sin(phi)*xa + cos(phi)*(-ya); theta1a; c2a] ];
end

%case b
b = sqrt(sum((c2b-c1).^2));
xb = (r1^2+b^2-r2^2)/(2*b);
sqr_yb = r1^2 - xb^2;
if (abs(sqr_yb) < myeps) %one solution for case a
  phi = atan2(c2b(2) - c1(2), c2b(1) - c1(1));
  s = [s, [c1(1) + cos(phi)*xb; c1(2) + sin(phi)*xb; theta1b; c2b] ];
elseif (sqr_yb > 0) %two solutions for case a
  yb = sqrt(sqr_yb);
  phi = atan2(c2b(2) - c1(2), c2b(1) - c1(1));
  s = [s, [c1(1) + cos(phi)*xb - sin(phi)*yb;    c1(2) + sin(phi)*xb + cos(phi)*yb;    theta1b; c2b], ...
          [c1(1) + cos(phi)*xb - sin(phi)*(-yb); c1(2) + sin(phi)*xb + cos(phi)*(-yb); theta1b; c2b] ];
end
%now calculate the angles
J3 = [];

%check if we have any solution
if (isempty(s))
  J = [NaN; NaN; NaN; NaN; NaN; NaN];
  return;
end

for i=1:size(s,2)
  theta2 = atan2(s(1,i) - c1(1), s(2,i) - c1(2)); 
  theta3 = atan2(s(4,i) - s(1,i), s(5,i) - s(2,i)) - theta2 + ...
    atan2(robot.denavitHartenberg(2,3) ,robot.denavitHartenberg(4,4));
  if (theta3 <= -pi); theta3 = theta3 + 2*pi; end
  if (theta3 >   pi); theta3 = theta3 - 2*pi; end
  
  %check if within limits
  theta1chk = s(3,i)*180/pi;
  theta2chk = theta2*180/pi;
  theta3chk = theta3*180/pi;
  
  if (theta1chk < robot.jointLimits(1,1) || theta1chk > robot.jointLimits(2,1)); continue; end
  if (theta2chk < robot.jointLimits(1,2) || theta2chk > robot.jointLimits(2,2)); continue; end
  if (theta3chk < robot.jointLimits(1,3) || theta3chk > robot.jointLimits(2,3)); continue; end
  
  J3 = [J3, [s(3,i); theta2; theta3] ]; %keep em in radians here
end

%calculate the last three joints
J = [];

for i=1:size(J3,2)

  %dkt for the three joints
  M = eye(4);
  
  for j=1:3
    M = M * denavitHartenberg(robot.denavitHartenberg(:,j) + ...
                J3(j,i) * robot.denavitHartenbergParameters(:,j));
  end
  
  %the translations in the fourth joints (only to check that W is
  %correctly, comment otherwise as it is not necessary)
  %M = M * [1 0 0 robot.denavitHartenberg(4,4); 0 1 0 0; 0 0 1 robot.denavitHartenberg(2,3); 0 0 0 1];
              
  mtx = M\T;
  c5 = mtx(3,3);

  %check if it is straight
  if (abs(c5 - 1) < myeps)
    
    theta4 = 0; % parameter, but I want it zero
    theta5 = 0;
    theta6 = atan2(-mtx(1,2), mtx(2,2)) - pi;
    
    if (theta6 <= -pi); theta6 = theta6 + 2*pi; end
    if (theta6 >   pi); theta6 = theta6 - 2*pi; end
    
    theta4 = theta4*180/pi;
    theta5 = theta5*180/pi;
    theta6 = theta6*180/pi;

    if (theta4 < robot.jointLimits(1,4) || theta4 > robot.jointLimits(2,4)); continue; end
    if (theta5 < robot.jointLimits(1,5) || theta5 > robot.jointLimits(2,5)); continue; end
    if (theta6 < robot.jointLimits(1,6) || theta6 > robot.jointLimits(2,6)); continue; end
  
    J = [J, [J3(:,i)*180/pi; theta4; theta5; theta6] ];
    
  elseif (abs(c5 + 1) < myeps)
    % discarded; the robot would try to put an arm into another (theta5
    % can't be equal to pi)
    continue;
  else
    c5 = max(c5, -1);
    c5 = min(c5, 1);
    
    %two solutions
    theta5a = acos(c5); 
    theta5b = -theta5a;
    
    sg5a = sign(sin(theta5a));
    sg5b = -sg5a;
    
    theta4a = atan2(-mtx(2,3) / sg5a, -mtx(1,3) / sg5a);
    theta6a = atan2(-mtx(3,2) / sg5a, mtx(3,1) / sg5a) - pi;
    
    theta4b = atan2(-mtx(2,3) / sg5b, -mtx(1,3) / sg5b);
    theta6b = atan2(-mtx(3,2) / sg5b, mtx(3,1) / sg5b) - pi;
    
    if (theta6a <= -pi); theta6a = theta6a + 2*pi; end
    if (theta6a >   pi); theta6a = theta6a - 2*pi; end
    if (theta6b <= -pi); theta6b = theta6b + 2*pi; end
    if (theta6b >   pi); theta6b = theta6b - 2*pi; end
    
    %put them into an array and convert to degrees
    angles = [ theta4a, theta4b;
               theta5a, theta5b;
               theta6a, theta6b ]*180/pi;
    
    %filter the 'b' and '2' case here
    for j=1:2   
      if (angles(1,j) < robot.jointLimits(1,4) || angles(1,j) > robot.jointLimits(2,4)); continue; end
      if (angles(2,j) < robot.jointLimits(1,5) || angles(2,j) > robot.jointLimits(2,5)); continue; end
      if (angles(3,j) < robot.jointLimits(1,6) || angles(3,j) > robot.jointLimits(2,6)); continue; end
      J = [J, [J3(:,i)*180/pi; angles(1,j); angles(2,j); angles(3,j)] ];    
    end
  end
    
end

if (isempty(J))
  J = [NaN; NaN; NaN; NaN; NaN; NaN];
  return;
end

end
