function [reply r] = mmMov(robot, type, pos, flags, args)
%MMMOV Moves the robot to specified position using joint interpolation.
%COMMONARGS r, robot
%
% [reply r] = mmMov(robot, type, pos [, flags, args]);
%
% Arguments:
%   type: the type of the coordinates (this can be 'P' for XYZ, 'J' for
%   joint and 'X' for XYZ coordinates and the rotation in joint coordinates)
%   pos:  column vector of the axis coordinates
%
% Optional Arguments:
%   flags:  flags for robot (see manual)
%   args:   any other arguments appended at the end
% 
% Return values:
%   reply:  reply message from the robot
%
% The movement inplementation is asynchronous. Use 'mmWaitForStop' in order
% to wait until the movement is complete (or error is thrown). XYZ
% coordinates are in milimetres, joint coordinates are in degradians.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    2     12.10.2010 16:10:32      Added option for specifying flags
% Martin Meloun                    1     30.4.2010 17:05:56       Initial revision

if (robot.secureMovement)
  if (robot.verifyMovement('J', type, pos))
    robot.throwErr('VERIFICATION_FAILED', 'Movement verification failed.');
    reply = 'VERIFICATION_FAILED';
    r = 1;
    return;
  end
end

if (nargin >= 4)
  coords = sprintf('%c1 = %s (%d,%d)',type,mmCreateCoords(pos),flags(1),flags(2));
else
  coords = sprintf('%c1 = %s',type,mmCreateCoords(pos));
end

[~, r1] = mmExec(robot, coords);
if (nargin == 5)
  [reply r2] = mmExec(robot, sprintf('MOV %c1 %s', type, args));
else
  [reply r2] = mmExec(robot, sprintf('MOV %c1', type));
end

r = r1 | r2;

end
