function robot = mmOpen(robotname, varargin)
%MMOPEN Opens the Melfa Robot Communication.
%
% robot = mmOpen(robotname, varargin)
%
% Opens a communication with a Melfa Robot.
% Varargin input are pairs 'propertyName', 'propertyValue', and those pairs are assigned to the robot.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    2     9.7.2010 14:09:47        Configures the control unit
% Martin Meloun                    1     30.4.2010 17:05:56       Initial revision

  robot = mmCreateRobot(robotname, varargin{:});

  % Low-level COM port IO
  eval(['robot.com=mmCom_', robot.comtype, ';']);
  robot.com.open( robot );

  % Set timeout in seconds
  robot.com.settimeout( 5 );

  % Reset the hardhome state
  if (robot.hhflag == 1)
    robot.hhstate = 0;
  elseif (robot.hhflag == 2)
    robot.hhstate = 1;
  end

  robot.verbose = 1;

  errStat = robot.thrError;
  robot.thrError = 1;

  %write it directly to com, for initialization
  mmAcquireCom(robot);
  mmPrintf(robot,'OPEN=NARCUSR');
  mmScanf(robot); %ignore the return value

  robot.thrError = errStat;

  %set base and tool
  %turn on control, if it fails, close the robot and throw an error
  r = mmCntlOn(robot);
  if (r)
    robot.errorType = 'INIT_FAIL';
    robot.errorMsg = sprintf('Could not turn on control.');
    robot.com.close();
    robot.com = [];
    fsprintf('Could not turn on control. . Closing.\n');
    return;
  end

  if (~isempty(robot.onSecureInit))
    r = robot.onSecureInit();
    if (r)
      robot.errorType = 'SECURE_INIT_FAIL';
      robot.errorMsg = sprintf('Could not initialize security.');
      robot.com.close();
      robot.com = [];
      fprintf('Could not initialize security. Closing.\n');
      return;
    end
  end

  [A B C] = mtxToAngles(robot.base);
  mmExec(robot, 'BASE (%.2f,%.2f,%.2f,%.2f,%.2f,%.2f)', ...
    robot.base(1,4), robot.base(2,4), robot.base(3,4), A, B, C);

  [A B C] = mtxToAngles(robot.tool);
  mmExec(robot, 'TOOL (%.2f,%.2f,%.2f,%.2f,%.2f,%.2f)', ...
    robot.tool(1,4), robot.tool(2,4), robot.tool(3,4), A, B, C);

  mmCntlOff(robot);

end
