function mmPlotRecordedMove(robot, recordData, dynAxisScaling, axisPts, staticUserPlot, dynamicUserPlot, userdata, newfig)
%MMPLOTRECORDEDMOVE Plots the robot recorded moving.
%COMMONARGS robot
%
% mmPlotRecordedMove(robot, recordData, [dynAxisScaling, axisPts, staticUserPlot, dynamicUserPlot, userdata, newfig]);
%
% Arguments:
%   recordData: structure obtained while recording move.
% 
% Optional Arguments:
%   dynAxisScaling:   Whether to scale axis dynamically based on current robot position
%   axisPts:          Points to include to dynamical axis scaling
%   staticUserPlot:   User plot for things not relevant to current robot position, args: (figure, robot, userdata).
%   dynamicUserPlot:  User plot for things relevant to current robot position, args: (figure, robot, coordinates, time, userdata).
%   newfig:           Do not create a new figure for plotting.
%
% This function traces the movement of robot from start point to end point
% in linear interpolation and plots it.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    2     18.10.2010 15:41:29      Added callbacks
% Martin Meloun                    1     9.7.2010 14:09:47        Initial revision

  if (nargin < 3)
    dynAxisScaling = 1;
  end
  
  if (nargin < 4)
    axisPts = [];
  end

  if (nargin < 5)
    staticUserPlot = [];
  end
  
  if (nargin < 6)
    dynamicUserPlot = [];
  end
  
  if (nargin < 7)
    userdata = [];
  end
  
  if (nargin < 8)
    newfig = 1;
  end
  
  if (newfig)
    data.figure = figure('Name', [robot.description ': Recorded movement'], 'NumberTitle', 'off', 'ToolBar', 'figure');
  else
    data.figure = get(0, 'CurrentFigure');
  end

  sz = size(recordData.jCoords, 2);
  data.jCoordinates = recordData.jCoords;
  data.times = recordData.times;
  data.trajectory = zeros(3, sz);
  data.dynAxisScaling = dynAxisScaling;
  data.axisPts = axisPts;
  data.staticCallback = staticUserPlot;
  data.dynamicCallback = dynamicUserPlot;
  data.userdata = userdata;

  for i=1:sz
    point = mmDkt(robot, data.jCoordinates(:,i));
    data.trajectory(:,i) = point(1:3);
  end

  data.robot = robot;
  data.rtrobot = mmrtCreateInteractiveRobot(robot);
  max = size(data.trajectory, 2) - 1;
  data.plotHandle=newplot;
  
  %these are the static handles
  mmPlotTrajectory(robot, data.trajectory, 0, 1, 0);
  
  if (~isempty(data.staticCallback))
    data.staticCallback(data.figure, data.robot, data.userdata);
  end
  
  data.staticPlots=get(data.plotHandle, 'Children');
  
  mmPlotRobot(data.robot, data.jCoordinates(:,1), 0, 1, 0, data.dynAxisScaling, data.axisPts, 0);
  
  data.timeLabel = uicontrol('Style','text','Units','Normalized','Position',[.35 .94 .3 .025], 'HorizontalAlignment', 'center', ...
    'String', sprintf('Time: %.3f s', recordData.times(1)));

  uicontrol('Style','slider','Units','Normalized','Position',[.35 .965 .3 .025], 'UserData', data, 'Max', max,...
    'Callback', @Callback, 'SliderStep', [1/(max+1) 10/(max+1)]);

  if (~isempty(data.dynamicCallback))
    data.dynamicCallback(data.figure, data.robot, data.jCoordinates(:,1), data.times(1), data.userdata);
  end

end

function Callback(obj, ~)

  value = round(get(obj, 'Value')); %0 - (size - 1)

  data = get(obj, 'UserData');

  set(0,'CurrentFigure', data.figure);
  set(data.timeLabel, 'String', sprintf('Time: %.3f s', data.times(value + 1)));
  
  jCoor = data.jCoordinates(:,value + 1);
  
  %clear everything except the slider and the robot 
  children=get(data.plotHandle, 'Children');
  for j=1:length(children);

    %something not to be redrawn
    if (any(data.staticPlots==children(j)))
      continue;
    end

    delete(children(j));
  end

  %Plot robot
  mmPlotRobot(data.robot, jCoor, 0, 1, 0, data.dynAxisScaling, data.axisPts, 0);
  
  if (~isempty(data.dynamicCallback))
    data.dynamicCallback(data.figure, data.robot, data.jCoordinates(:,value + 1), data.times(value + 1), data.userdata);
  end

end
