function mmPlotRobot(robot, jointCoordinates, logScale, simple, baseAxes, dynAxisScaling, axisPts, newfig)
%MMPLOTROBOT Plots the robot rigid body with specified coordinates.
%COMMONARGS robot
%
% mmPlotRobot(robot, jointCoordinates [, logScale, simple, baseAxes, dynAxisScale, axisPts, newfig]);
%
% Arguments:
%   jointCoordinates: 6x1 vector, coordinates for the joints
% 
% Optional Arguments:
%   logScale:       Logatimic scale is used. Off by default.
%   simple:         Plot just a simple model of the robot in current position.
%   baseAxes:       Plot axes of base
%   dynAxisScaling: Whether to scale axis dynamically based on current robot position (only for simple plot)
%   axisPts:        Points to include to dynamical axis scaling
%   newfig:         Do not create a new figure for plotting. Only valid if simple is on
%
% This function draws a schema for the robot in desired joint coordinates.
% There is also the posibility to use logaritmic scale.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    4     18.7.2011 14:54:38       Improved drawing
% Martin Meloun                    3     17.10.2010 15:31:54      Fixed Joint 4 Splitting
% Martin Meloun                    2     9.7.2010 14:09:47        Added more parameters and simplified version
% Martin Meloun                    1     25.5.2010 15:28:05       Initial revision

  if (nargin < 3)
    logScale = 0;
  end

  if (nargin < 4)
    simple = 0;
  end

  if (nargin < 5)
    baseAxes = 0;
  end
  
  if (nargin < 6)
    dynAxisScaling = 1;
  end
  
  if (nargin < 7)
    axisPts = [];
  end
  
  if (nargin < 8)
    newfig = 1;
  end

  if (~newfig && ~simple)
    error('Cannot plot non-simplified skeleteon into existing figure.')
  end

  if (newfig)
    if (simple)
      figure('Name', robot.description, 'NumberTitle', 'off');
    else
      figure('Name', [robot.description ': Part A'], 'NumberTitle', 'off');
    end
  end

  T = robot.base;
  A = T * [0; 0; 0; 1];
  jointCoordinates = jointCoordinates*pi/180; %deg to rad
  detailFigure=0;
  
  jointAxisLen = 1.2;
  jointRotRadius = 0.8;
  axisLenSmall = 3;
  axisLenMedium = 4;
  axisLenBig = 5;
  arrowScale = 1;
  textShiftScale = 1;

  if (~logScale)
    scaleMultiplier = 33;
    jointAxisLen = jointAxisLen * scaleMultiplier;
    jointRotRadius = jointRotRadius * scaleMultiplier;
    axisLenSmall = axisLenSmall * scaleMultiplier;
    axisLenMedium = axisLenMedium * scaleMultiplier;
    axisLenBig = axisLenBig * scaleMultiplier;
    arrowScale = arrowScale * scaleMultiplier;
    textShiftScale = textShiftScale * scaleMultiplier;
  end
  
  [xcz,ycz,zcz] = cylinder(axisLenSmall/4, 8);
  zcz(1,:) = -axisLenSmall/2;
  zcz(2,:) = axisLenSmall/2;
  xyzcz = [xcz(:)'; ycz(:)'; zcz(:)'; ones(1,2*8+2)]; 

  [ycx,zcx,xcx] = cylinder(axisLenSmall/4, 8);
  xcx(1,:) = -axisLenSmall/2;
  xcx(2,:) = axisLenSmall/2;
  xyzcx = [xcx(:)'; ycx(:)'; zcx(:)'; ones(1,2*8+2)]; 
  
  hold on
  xlabel('X [mm]');
  ylabel('Y [mm]');
  zlabel('Z [mm]');
    
  if (baseAxes)
    plotAxes(T, axisLenMedium, arrowScale, textShiftScale, 'k', 'B', '');
  end
  
  plot3(A(1), A(2), A(3), '*b', 'LineWidth', 6);
    
  DenavitHartenberg = robot.denavitHartenberg + ...
    ([jointCoordinates'; jointCoordinates'; jointCoordinates'; jointCoordinates']) ...
    .* robot.denavitHartenbergParameters; 

  splitJoint4 = 0;
  splittedJoint4 = 0;
  xRotJoint4 = 0;
  xTransJoint4 = 0;
  zRotJoint4 = 0;

  i = 1;
  sz = size(DenavitHartenberg, 2);

  if (simple)
    grid on;
    axisPts = [zeros(3,sz*2+1), axisPts];
    axisPts(:,sz*2+1) = A(1:3);
  end
  
  while (i <= sz)
    T1 = T;
    DH = DenavitHartenberg(:,i);

    %normalize it for plotting
    if (logScale)
      %or logaritmize the distances
      DH(2) = log(1 + abs(DH(2))) * sign(DH(2));
      DH(4) = log(1 + abs(DH(4))) * sign(DH(4));
    end

    %on detail figure plot the start point as well
    if (detailFigure)
      plot3(A(1), A(2), A(3), '*b', 'LineWidth', 6);
    end
    
    %translate in z axis - first plot it to this point
    %it is irrelevant wheter rotation or translation over the axis is done
    %first, for plotting, translating first is better
    B = T1 * [0; 0; 0; 1];
    patch([A(1) B(1)], [A(2) B(2)], [A(3) B(3)], [0 0], 'EdgeColor', 'black', 'LineWidth', 3, 'EdgeAlpha', 0.5);
    A = B;

    %plot the new point
    trans = mtxTranslate([0; 0; DH(4)]);
    T1 = T1 * trans;
    B = T1 * [0; 0; 0; 1];
    patch([A(1) B(1)], [A(2) B(2)], [A(3) B(3)], [0 0], 'EdgeColor', 'black', 'LineWidth', 3, 'EdgeAlpha', 0.5);
    plot3(B(1), B(2), B(3), '*b', 'LineWidth', 6);
    
    if (simple)
      axisPts(:, 2*i-1) = B(1:3);
    end

    if ((DH(4) ~= 0) && (~simple))
      TXT = ((A + B) / 2) + [0.2*textShiftScale; 0.2*textShiftScale; 0.2*textShiftScale; 0];
      text(TXT(1), TXT(2), TXT(3), sprintf('d_%d', i));
    end

    A = B;

    %rotate over z axis - plot X axis so the angle can be seen
    if ((DH(3) ~= 0 || (jointCoordinates(i) ~= 0 && robot.denavitHartenbergParameters(3,i) ~= 0)) && ~simple) 
      %plot separately the offset and the parameter
      plotAxisLine(T1, 'X', 0, jointAxisLen, 'b');
      plotRotation(T1, arrowScale, 'Z', 0, mod(robot.denavitHartenberg(3,i), 2*pi), jointRotRadius, 'b');

      if (robot.denavitHartenbergParameters(3,i) ~= 0)
        T2 = T1 * mtxRotateZ(robot.denavitHartenberg(3,i));
        plotAxisLine(T2, 'X', 0, jointAxisLen, 'b');
        plotRotation(T2, arrowScale, 'Z', 0, jointCoordinates(i), jointRotRadius, 'r', sprintf('{\\theta}_%d', i));
      end

      T1 = T1 * mtxRotateZ(DH(3));
      plotAxisLine(T1, 'X', 0, jointAxisLen, 'b');
      if (DH(1) ~= 0)
        %plot the 'fake' rotation over y for consistency
        plotRotation(T1, arrowScale, 'Y', pi, -pi/2, jointRotRadius, 'm');
      end
    elseif (simple && robot.denavitHartenbergParameters(3, i) ~= 0)
      %plot rotation axis and cylinder
      plotAxisLine(T1, 'Z', -axisLenBig/2, axisLenBig/2, 'r', 1);
      %and cylinder
      xyzt = T1 * xyzcz;
      ncols = numcols(xyzt)/2;
      xt = reshape(xyzt(1,:), 2, ncols);
      yt = reshape(xyzt(2,:), 2, ncols);
      zt = reshape(xyzt(3,:), 2, ncols);
      surface(xt,yt,zt,'FaceColor', 'blue', 'FaceAlpha', 0.33);
      T1 = T1 * mtxRotateZ(DH(3));
    else
      T1 = T1 * mtxRotateZ(DH(3));
    end

    %rotate over new x axis - plot Z axis so the angle can be seen
    if ((DH(1) ~= 0 || (jointCoordinates(i) ~= 0 && robot.denavitHartenbergParameters(1,i) ~= 0)) && ~simple) 
      plotAxisLine(T1, 'Z', 0, jointAxisLen, 'b');
      plotRotation(T1, arrowScale, 'X', pi/2, mod(robot.denavitHartenberg(1,i), 2*pi), jointRotRadius, 'b');

      if (robot.denavitHartenbergParameters(1,i) ~= 0)
        T2 = T1 * mtxRotateX(robot.denavitHartenberg(1,i));
        plotAxisLine(T2, 'Z', 0, jointAxisLen, 'b');
        plotRotation(T2, arrowScale, 'X', pi/2, jointCoordinates(i), jointRotRadius, 'r', sprintf('{\\alpha}_%d', i));
      end

      T1 = T1 * mtxRotateX(DH(1));
      plotAxisLine(T1, 'Z', 0, jointAxisLen, 'b');
    elseif (simple && robot.denavitHartenbergParameters(1, i) ~= 0)
      %plot rotation axis
      plotAxisLine(T1, 'X', -axisLenBig/2, axisLenBig/2, 'r', 1);
      %and cylinder
      xyzt = T1 * xyzcx;
      ncols = numcols(xyzt)/2;
      xt = reshape(xyzt(1,:), 2, ncols);
      yt = reshape(xyzt(2,:), 2, ncols);
      zt = reshape(xyzt(3,:), 2, ncols);
      surface(xt,yt,zt,'FaceColor', 'blue', 'FaceAlpha', 0.33);
      T1 = T1 * mtxRotateX(DH(1));
    else
      T1 = T1 * mtxRotateX(DH(1));
    end

    %translate in new x axis - first plot it to this point
    B = T1 * [0; 0; 0; 1];
    patch([A(1) B(1)], [A(2) B(2)], [A(3) B(3)], [0 0], 'EdgeColor', 'black', 'LineWidth', 3, 'EdgeAlpha', 0.5);
    A = B;

    %plot the new point
    trans = mtxTranslate([DH(2); 0; 0]);
    T1 = T1 * trans;
    B = T1 * [0; 0; 0; 1];
    patch([A(1) B(1)], [A(2) B(2)], [A(3) B(3)], [0 0], 'EdgeColor', 'black', 'LineWidth', 3, 'EdgeAlpha', 0.5);
    plot3(B(1), B(2), B(3), '*b', 'LineWidth', 6);

    if (simple)
      axisPts(:, 2*i) = B(1:3);
    end
    
    if ((DH(2) ~= 0) && (~simple))
      TXT = ((A + B) / 2) + [0.2*textShiftScale; 0.2*textShiftScale; 0.2*textShiftScale; 0];
      text(TXT(1), TXT(2), TXT(3), sprintf('a_%d', i));
    end

    A = B;    
    T = T * denavitHartenberg(DH);

    %plotting options
    if ((i == 3 || ((i == 4) && splitJoint4)) && (~simple))

      %split joint 4
      if (i == 3 && (DenavitHartenberg(4,2) ~= 0 || DenavitHartenberg(4,4) ~= 0) )
        splitJoint4 = 1;
        splittedJoint4 = 1;
        
        %zTranslation is always in the main graph
        zRotJoint4 = DenavitHartenberg(3,4);
        DenavitHartenberg(3,4) = 0;
        
        %xTranslation is in the main graph, if the zRotation isn't
        %parametric
        if(robot.denavitHartenbergParameters(4,3) == 0)
          xRotJoint4 = DenavitHartenberg(1,4);
          xTransJoint4 = DenavitHartenberg(2,4);
          DenavitHartenberg(1,4) = 0;
          DenavitHartenberg(2,4) = 0;
        end 
      else
        if (i == 4)
          plotAxes(T, axisLenMedium, arrowScale, textShiftScale, 'k', 'W',  '_3''');
          DenavitHartenberg(1,4) = xRotJoint4;
          DenavitHartenberg(2,4) = xTransJoint4;
          DenavitHartenberg(3,4) = zRotJoint4;
          DenavitHartenberg(4,4) = 0;
          splitJoint4 = 0;
          i = i -1;
          title('Joints 1 - 3, 4 (translations only)');
          hold off;
        else
          plotAxes(T, axisLenMedium, arrowScale, textShiftScale, 'k', 'W',  '_3');
          title('Joints 1 - 3');
          hold off;
        end

        %new figure
        detailFigure=1;
        figure('Name',[robot.description ': Part B'], 'NumberTitle', 'off');
        subplot(221);
        hold on;
        xlabel('X');
        ylabel('Y');
        zlabel('Z');
        plotAxes(T, axisLenSmall, arrowScale, textShiftScale, 'k', '', sprintf('_%d', i));
      end
    elseif ((i > 3 && i <= 6) && (~simple))
      plotAxes(T, axisLenBig, arrowScale, textShiftScale, 'k', '', sprintf('_%d', i));

      if (splittedJoint4 && i == 4)
        title('Joint 4 (rotation only)');
      else
        title(sprintf('Joint %d', i));
      end
      hold off;
      subplot(2, 2, i-2);
      hold on;
      xlabel('X');
      ylabel('Y');
      zlabel('Z');
      plotAxes(T, axisLenSmall, arrowScale, textShiftScale, 'k', '', sprintf('_%d', i));
      if (i == 6)
        plotToolToLastSubplot = 1;
      end   
    end

    i = i + 1;
  end

  %plot tool
  if ((~simple) && (~plotToolToLastSubplot))
    hold off;
    subplot(224);
    hold on;
  end

  A = T * [0; 0; 0; 1];
  RT = robot.tool;

  if (logScale)
    %or logaritmize the distances
    RT(1:3,4) = log(1 + abs(RT(1:3,4))) .* sign(RT(1:3,4));
  end

  %translate it
  Tt = T * mtxTranslate(RT(1:3, 4));
  B = Tt * [0; 0; 0; 1];
  %check if worthy plotting
  if (sum(abs(B-A)) > 0.001)
    patch([A(1) B(1)], [A(2) B(2)], [A(3) B(3)], [0 0], 'EdgeColor', 'black', 'LineWidth', 3, 'EdgeAlpha', 0.5);
    if (~simple)
      plot3(A(1), A(2), A(3), '*b', 'LineWidth', 6);
      plot3(B(1), B(2), B(3), '*b', 'LineWidth', 6);
      tp = A + (B - A) / 2;
      text(tp(1), tp(2), tp(3), 'd_t');
    else
      plot3(B(1), B(2), B(3), '*b', 'LineWidth', 6);
    end
  end

  if (~simple)
    %get the rotations
    [angles] = mtxToAngles(RT)/180*pi;

    if (abs(angles(3)) > 0.01)
      plotAxisLine(Tt, 'X', 0, jointAxisLen, 'b');
      plotRotation(Tt, arrowScale, 'Z', 0, angles(3), jointRotRadius, 'b', sprintf('{\\gamma}_t'));
      Tt = Tt * mtxRotateZ(angles(3));
      plotAxisLine(Tt, 'X', 0, jointAxisLen, 'b');    
    end

    if (abs(angles(2)) > 0.01)  
      plotAxisLine(Tt, 'X', 0, -jointAxisLen, 'b');
      plotRotation(Tt, arrowScale, 'Y', 0, angles(2), jointRotRadius, 'b', sprintf('{\\beta}_t'));
      Tt = Tt * mtxRotateY(angles(2));
      plotAxisLine(Tt, 'X', 0, -jointAxisLen, 'b');
    end

    if (abs(angles(1)) > 0.01)
      plotAxisLine(Tt, 'Y', 0, jointAxisLen, 'b');
      plotRotation(Tt, arrowScale, 'X', 0, angles(1), jointRotRadius, 'b', sprintf('{\\alpha}_t'));
      Tt = Tt * mtxRotateX(angles(1));
      plotAxisLine(Tt, 'Y', 0, jointAxisLen, 'b');
    end

    title('Tool');
  end

  T = T * RT;
  
  if (simple)
    plotAxes(T, axisLenMedium, arrowScale, textShiftScale, 'rgb', 'T', '_t');
    
    %axes
    if (dynAxisScaling)
      
      amin = min(axisPts')';
      amax = max(axisPts')';
      adiff = abs(amax-amin);
      mdiff = max(adiff);
      aadd = (mdiff - adiff)/2;

      %add border (100 pts)
      if (logScale)
        border = 3;
      else
        border = 100;
      end

      amin = amin - aadd - border;
      amax = amax + aadd + border;
      axis([amin(1) amax(1) amin(2) amax(2) amin(3) amax(3)]);
    
    else
      
      xlim = get(gca, 'XLim');
      ylim = get(gca, 'YLim');
      
      amin(1) = xlim(1);
      amin(2) = ylim(1);
      
      amax(1) = xlim(2);
      amax(2) = ylim(2);
      
    end
    
    %plot XY plane surface
    surface([amin(1) amax(1)], [amin(2) amax(2)], [0 0; 0 0], 'FaceColor', [0.66 0.66 0.66], 'FaceAlpha', 0.5);
    
  else
    plotAxes(T, axisLenMedium, arrowScale, textShiftScale, 'k', 'T', '_t');
  end
  
  hold off

end


function plotAxes(mtx, axisLen, arrowScale, textShiftScale, color, letter, tag)

  %color
  if (length(color) == 1)
    color = [color color color];
  end

  %center
  W = mtx*[0; 0; 0; 1];
  W = W(1:3); 
  
  %axis x with arrow
  X = mtx*[axisLen; 0; 0; 1];
  X = X(1:3);

  Xa = mtx*[axisLen - 0.5 * arrowScale; 0.2 * arrowScale; 0; 1];
  Xa = Xa(1:3);

  Xb = mtx*[axisLen - 0.5 * arrowScale; -0.2 * arrowScale; 0; 1];
  Xb = Xb(1:3);

  %axis y with arrow
  Y = mtx*[0; axisLen; 0; 1];
  Y = Y(1:3);

  Ya = mtx*[0.2 * arrowScale; axisLen - 0.5 * arrowScale; 0; 1];
  Ya = Ya(1:3);

  Yb = mtx*[-0.2 * arrowScale; axisLen - 0.5 * arrowScale; 0; 1];
  Yb = Yb(1:3);

  %axis z with arrow
  Z = mtx*[0; 0; axisLen; 1];
  Z = Z(1:3);

  Za = mtx*[0.2 * arrowScale; 0; axisLen - 0.5 * arrowScale; 1];
  Za = Za(1:3);

  Zb = mtx*[-0.2 * arrowScale; 0; axisLen - 0.5 * arrowScale; 1];
  Zb = Zb(1:3);

  %plot em
  plot3([W(1) X(1)], [W(2) X(2)], [W(3) X(3)], ['-' color(1)]);
  plot3([Xa(1) X(1) Xb(1)], [Xa(2) X(2) Xb(2)], [Xa(3) X(3) Xb(3)], ['-' color(1)]);
  plot3([W(1) Y(1)], [W(2) Y(2)], [W(3) Y(3)], ['-' color(2)]);
  plot3([Ya(1) Y(1) Yb(1)], [Ya(2) Y(2) Yb(2)], [Ya(3) Y(3) Yb(3)], ['-' color(2)]);
  plot3([W(1) Z(1)], [W(2) Z(2)], [W(3) Z(3)], ['-' color(3)]);
  plot3([Za(1) Z(1) Zb(1)], [Za(2) Z(2) Zb(2)], [Za(3) Z(3) Zb(3)], ['-' color(3)]);

  %text em
  text(W(1) - 0.4 * textShiftScale, W(2) - 0.4 * textShiftScale, W(3) - 0.4 * textShiftScale, letter);
  text(X(1) + 0.2 * textShiftScale, X(2) + 0.2 * textShiftScale, X(3) + 0.2 * textShiftScale, ['x' tag]);
  text(Y(1) + 0.2 * textShiftScale, Y(2) + 0.2 * textShiftScale, Y(3) + 0.2 * textShiftScale, ['y' tag]);
  text(Z(1) + 0.2 * textShiftScale, Z(2) + 0.2 * textShiftScale, Z(3) + 0.2 * textShiftScale, ['z' tag]);

end

function plotAxisLine(mtx, axis, axisStart, axisEnd, color, pline)

  if (nargin < 6)
    pline = 0;
  end

  %axis
  if (axis == 'X')
    W = mtx*[axisStart; 0; 0; 1];
    T = mtx*[axisEnd; 0; 0; 1];
  elseif (axis == 'Y')
    W = mtx*[0; axisStart; 0; 1];
    T = mtx*[0; axisEnd; 0; 1];
  elseif (axis == 'Z')
    W = mtx*[0; 0; axisStart; 1];
    T = mtx*[0; 0; axisEnd; 1];
  else
    error('Invalid axis specified.');
  end

  T = T(1:3);

  %plot it
  if (pline)
    patch([W(1) T(1)], [W(2) T(2)], [W(3) T(3)], [0 0], 'EdgeColor', color, 'LineWidth', 1.5, 'EdgeAlpha', 0.5);
  else
    plot3([W(1) T(1)], [W(2) T(2)], [W(3) T(3)], ['-' color]);
  end

end

function plotRotation(mtx, arrowScale, axisAround, offset, angle, rad, color, note)

  if (angle < 0)
    angle = mod(angle, 2*pi) - 2*pi;
  else
    angle = mod(angle, 2*pi);
  end

  if (abs(angle) < 0.05) %do not plot anything then
    return;
  end

  if (angle < 0)
    t = angle:0.05:0;
  else
    t = 0:0.05:angle;
  end

  t = t + offset;

  xp = rad*cos(t);
  yp = rad*sin(t);
  len = length(t);

  if (axisAround == 'Z')
    norm = eye(4);
  elseif (axisAround == 'Y')
    norm = [-1 0 0 0; 0 0 1 0; 0 1 0 0; 0 0 0 1];
  elseif (axisAround == 'X')
    norm = [0 0 1 0; 1 0 0 0; 0 1 0 0; 0 0 0 1];
  else
    error('Invalid axis specified');
  end

  nmtx = mtx * norm;
  A = nmtx * [xp(1); yp(1); 0; 1];

  for i=2:len
    %plot it
    B = nmtx * [xp(i); yp(i); 0; 1];
    plot3([A(1) B(1)], [A(2) B(2)], [A(3) B(3)], ['-' color]);
    A = B;
  end

  %plot the arrow
  armtx = mtx * norm * mtxRotateZ(offset + angle);
  Ar = armtx*[rad; 0; 0; 1];
  Ar1 = armtx*[rad - 0.1*arrowScale; -0.2*sign(angle)*arrowScale; 0; 1];
  Ar2= armtx*[rad + 0.1*arrowScale; -0.2*sign(angle)*arrowScale; 0; 1];


  plot3([Ar1(1) Ar(1) Ar2(1)], [Ar1(2) Ar(2) Ar2(2)], [Ar1(3) Ar(3) Ar2(3)], ['-' color]);
  xtp = rad*1.2*cos(angle/2 + offset);
  ytp = rad*1.2*sin(angle/2 + offset);
  tpos = nmtx*[xtp; ytp; 0; 1];
  tpos = tpos(1:3);

  if (nargin > 7)
    text('String',note,'Position',tpos);
  end

end
