classdef mmRobotDef < hgsetget
%MMROBOTDEF Base class for all the robots, they should inherit this class.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    1     30.4.2010 17:05:56       Initial revision
  
  properties

    % Communication options
    % ------------------------------------------------------------------------------

    com;                              % Communication object (TODO: fix some interface)
    comtype;                          % Communication type (Serial)
    comprops;                         % Communication properties
    portname;                         % Default port name
    controller;                       % Controller (probably CR2B-574 (the box))
    robotNo;                          % Robot No.
    slotNo;                           % Slot No.
    topVariableSign;                  % Unknown behaviour when reading values
    
    % Debugging
    % ------------------------------------------------------------------------------

    verbose;                          % Debugging
    robotError;                       % Robot is in error state (-1 for mmError), it is automatically refreshed each time you read state
    
    % Error handling (!!!ONLY BETWEEN THE CONTROLLER AND MATLAB!!!)
    % ------------------------------------------------------------------------------

    errorMsg;                         % The error message
    errorType;                        % The error type
    thrError;                         % Specifies if error is to be thrown or just 
                                      % saved to error variables                                                                 
    % Hard Homing
    % ------------------------------------------------------------------------------

    % Hard Homing 
    % Flag:
    % 0 - unsupported (done externally)
    % 1 - requires hardhoming (controlled from MATLAB)
    % 2 - absolute position reading / automated hardhoming (not controlled from MATLAB)
    hhflag; 

    % Hard Home State:
    % 0 - not hardhomed
    % 1 - hardhomed
    hhstate;

    % User
    % ------------------------------------------------------------------------------

    userdata;                       % Anything the user wishes is here
        
    
  end

  properties (SetAccess = protected )

    % Names
    % ------------------------------------------------------------------------------

    name;                           % Robot name
    description;                    % Robot description
    series;                         % Robot series
    pincode;                        % Verification pincode which is checked with the controller unit when opened.
                                    % The purpose of this is only to prevent using the wrong Matlab robot structure on the current setup. 
                                    % If the pin in the controller is 0 anything is accepted. (not implemented yet)
                                    
    % Properties
    % ------------------------------------------------------------------------------

    DOF;                            % Degrees of freedom
    joints;                         % Joint types (example 'RRRRRR')
    denavitHartenberg;              % Denavit-Hartenberg notation in form [alpha; a; theta; d] (n columns)
                                    % Alpha is rotation over new x axis
                                    % a is translation in new x axis
                                    % Theta is rotation over z axis
                                    % d is translation in z axis
                                    % Order:
                                    % Rz(theta)*Tz(D)*Tx(a)*Rx(alpha)
                                    % Do not make one notation for a pair
                                    % of joints (like revolute + prismatic)
                                    % use radians here
    denavitHartenbergParameters;    % Vector how to apply parameters (0 nothing, 1 apply)
                                    % Example for parameter being theta: [0; 0; 1; 0]
    base;                           % Transformation Matrix for Base
    tool;                           % Transformation Matrix for Tool
    jointLimits;                    % Limits for the joints
    A76;                            % For Inverse kinematic task; to obtain the point
                                    % for the first three joints
    ikt;                            % Reference to the inverse kinematics task function
    testKinematics;                 % Reference to the function to test dkt and ikt
    getflags;                       % Reference to the get flags function

    % Safety
    % ------------------------------------------------------------------------------   
    
    safetyData;                     % Open structure to contain any data to safety
    secureMovement;                 % Whether to verify the movement                
    secureSpeed;                    % Whether to verify the speed                         
    secureAccel;                    % Whether to verify the acceleration and deceleration. 
                                    
    % Verifiers
    % Return value is 'r' type (0 - success, anything else error)
    % ------------------------------------------------------------------------------
    
    % Prototype " function r = verifyMovement(robot, interpolation, type, pos) ".
    % interpolation is 'J' for joint and 'P' for linear
    % type, pos are arguments from mmMov function
    verifyMovement; 
    
    % Prototype " function r = verifySpeed(robot, spdtype, speed) ".
    % spd_type is SPDTYPE_BASE for base speed, SPDTYPE_INTERPOLATION for interpol. speed, SPDTYPE_JOG for jog speed.
    verifySpeed;

    % Function pointer to " function r = verifyAccel(robot, accel, decel)
    verifyAccel;                    
    
    % Events (protected)
    % ------------------------------------------------------------------------------
    
    % Prototype " function r = onSecureInit(robot) ".
    % Initialize the properties to match security or throw error to terminate the connection.
    % Typically set the parameters into the range.
    onSecureInit;
end

  methods 
    % ------------------------------------------------------------------------------  
    function throwErr(robot, type, msg, varargin)
    %MMROBOTDEF/THROWERR  Throws an error.
    %
    % robot.throwErr(type, msg);
    %
    % Throws an error (both arguments are string). If thrError is set to 1
    % an error is thrown into the workspace.

      robot.errorType = type;
      robot.errorMsg = msg;
      
      if (robot.thrError)
        error('TYPE=%s, MSG=%s', type, sprintf(msg, varargin{:}));
      end
      
    end %func
    
    % ------------------------------------------------------------------------------  
    function clearErr(robot)
    %MMROBOTDEF/CLEARERR  Clears error noted in the robot properties.
    %
    % robot.clearErr();
    %
    % Clears the error in the robot properties. Does not call purge.

      robot.errorType = [];
      robot.errorMsg = [];
    end %func

    % ------------------------------------------------------------------------------  
    function b = isErr(robot)
    %MMROBOTDEF/ISERR  Checks if there is an error in robot properties
    %
    % b = robot.isErr();

      b = ~(isempty(robot.errorType));
    end %func
  end  %methods 
end %cls

