function jointTrajectory = mmSimulateMove(robot, type, positionJCoor, coordType, coordinates, noPoints)
%MMSIMULATEMOVE Simulates movement from A to B.
%COMMONARGS robot
%
% mmSimulateMove(robot, type, positionJCoor, coordType, coordinates [, noPoints]);
%
% Arguments:
%   type: interpolation - J for joint, P for linear
%   positionJCoor: 6x1 vector, start point in joint coordinates.
%   coordType: coordinates type - J for joint, P for linear
%   coordinates:   6x1 vector, of the coordinates type specified in previous parameter 
% 
% Optional Arguments:
%   noPoints: Number of the points of the simulated move.
%
% Return Value:
%   jointTrajectory: The trajectory the robot in the movement.
%
% This function simulates the movement between two points in desired
% interpolation. Simulates only the trajectory, not time.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    1     12.10.2010 16:34:03      Initial revision

%we don't really care here about the resolution
%secondly the movement isn't checked for verification

  if (type == 'J')
    %part of TraceJoint, but just the interpolation
    if (nargin < 6)
      jdist = sqrt(sum((coordinates - positionJCoor).^2)); % in halves of degradians
      noPoints = round(jdist);
    end

    if (coordType == 'J')
      endJCoor = coordinates;
      %test if end coordinates are within the limits
      if ((sum(endJCoor < (robot.jointLimits(1,:)')) +  sum(endJCoor > (robot.jointLimits(2,:)'))) > 0 )
        jointTrajectory = [];
        return;
      end
    elseif (coordType == 'P')
      endJCoor = mmIkt(robot, coordinates);

      if (isnan(endJCoor(1,1)))
        jointTrajectory = [];
        return;
      end

      endJCoor=endJCoor(:,1);

    else
      error('Unknown coordinate type.');
    end

    jointTrajectory = zeros(6, noPoints + 1);
    step = (endJCoor - positionJCoor) / noPoints;

    for i=1:noPoints+1    
      jointTrajectory(:, i) = positionJCoor + (step * (i-1));
    end
  elseif (type == 'P')
    %equal to TraceLinear
    if (nargin < 6)
      if (coordType == 'J')

        %test if end coordinates are within the limits
        if ((sum(coordinates < (robot.jointLimits(1,:)')) +  sum(coordinates > (robot.jointLimits(2,:)'))) > 0 )
          jointTrajectory = [];
          return;
        end

        target=mmDkt(robot, coordinates);
      elseif (coordType == 'P')
        target=coordinates;
      else
        error('Unknown coordinate type.');
      end

      jointTrajectory = mmTraceLinear(robot, positionJCoor, target);
    else

      if (coordType == 'J')

        %test if end coordinates are within the limits
        if ((sum(coordinates < (robot.jointLimits(1,:)')) +  sum(coordinates > (robot.jointLimits(2,:)'))) > 0 )
          jointTrajectory = [];
          return;
        end

        target=mmDkt(robot,coordinates);
      elseif (coordType == 'P')
        target=coordinates;
      else
        error('Unknown coordinate type.');
      end

      jointTrajectory = mmTraceLinear(robot, positionJCoor, target, noPoints);
    end 
  else
    error('Unknown move type.');
  end

end
