function jointTrajectory = mmTraceLinear(robot, positionJCoor, coordinates, noPoints)
%MMTRACELINEAR Traces the robot moving from A to B position in linear interp.
%COMMONARGS robot
%
% mmTraceLinear(robot, positionJCoor, coordinates [, noPoints]);
%
% Arguments:
%   positionJCoor: the robot position in joint coordinates
%   coordinates:   end point in [x; y; z; A; B; C] - Position and rotation column vector
% 
% Optional Arguments:
%   noPoints: If specified, the number of points calculated. Otherwise
%     estimated.
%
% Return Value:
%   6xN matrix with the joint coordinates of the movement from start to end
%
% This function traces the movement of robot from start point to end point
% in linear interpolation and returns the joint coordinates of the robot
% moving along the line.
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    1     9.7.2010 14:09:47        Initial revision

position = mmDkt(robot, positionJCoor);

if (nargin < 4)
  dist = sqrt(sum((coordinates(1:3) - position(1:3))).^2); % in milimetres
  noPoints = round(dist);
end

jointTrajectory = zeros(6, noPoints + 1);
delta=(coordinates - position);

if (noPoints < 1)
  step = 0;
else
  step = delta / noPoints;
end

%ensure there is no periodicity
for k=4:6
  if (abs(abs(delta(k)) - 360) < eps*10^6)
    step(k)=0;
  end
end

lastJCoor = positionJCoor;
tolerance = 2;

for i=1:noPoints+1
  
  JC = mmIkt(robot, position + (step * (i-1)));
  
  if (isnan(JC(1,1)))
    jointTrajectory = [];
    return;
  end
  
  sz = size(JC, 2);
  pass = 0;
  
  for j=1:sz
    %check if in tolerance limits 
    %(to ensure the robot doesn't change configuration rapidly)
    test = sum(abs(lastJCoor - JC(:, j)) > tolerance);
    
    if (test == 0)
      pass = 1;
      jointTrajectory(:, i) = JC(:, j);
      lastJCoor = JC(:, j);
      break;
    end
  end
    
  if (pass == 0)    
    
    %check if periodic solution can help
    for j2=1:sz

      %check for +360 available
      if (JC(6,j2) + 360 < robot.jointLimits(2,6));
        jcoords=JC(:,j2);
        jcoords(6)=jcoords(6)+360;
        
        %check if in tolerance limits 
        %(to ensure the robot doesn't change configuration rapidly)
        test = sum(abs(lastJCoor - jcoords)) > tolerance;

        if (test == 0)
          pass = 1;
          jointTrajectory(:, i) = jcoords;
          lastJCoor = jcoords;
          break;
        end
        
      end
      
      %check for -360 available
      if (JC(6,j2) - 360 > robot.jointLimits(1,6));
        jcoords=JC(:,j2);
        jcoords(6)=jcoords(6)-360;
        
        %check if in tolerance limits 
        %(to ensure the robot doesn't change configuration rapidly)
        test = sum(abs(lastJCoor - jcoords)) > tolerance;

        if (test == 0)
          pass = 1;
          jointTrajectory(:, i) = jcoords;
          lastJCoor = jcoords;
          break;
        end
      end
    end
    
    %check again if periodicity helped
    if (pass == 0)  
      jointTrajectory = [];
      return;
    end

  end
  
end

end
