function mmrtRecordingStart(robot, livePlot, plotUpdateFrequency, externalCallback, externalPlotCallback)
%MMRTRECORDINGSTART Starts robot movement recording. The samples must be added manually.
%COMMONARGS robot
%
% mmRecordingStart(robot, livePlot, [, externalCallback , externalPlotCallback]);
%
% Optional Arguments:
%   livePlot:             whether to plot the robot as it's being recorded
%   plotUpdateFrequency:  frequency of plot updating (default 4)
%   externalCallback:     prototype: function externalCallback(robot) - called when a sample is taken
%   externalPlotCallback: prototype: function externalPlotCallback(robot) - called when the scene is rendered
%
% This function starts recording the robot movement, however only
% initializes the structures and does the plotting. The samples must be
% added manually, to prevent flooding the serial line. You can use the
% mmRecordTillStop function or mmRecordingTakeSample function to add samples.
% The data are stored into the following structure:
% robot.userdata.tracing.trajectory.jCoords: recorded joint coordinates
% robot.userdata.tracing.trajectory.vals: calculated trajectory in XYZ-ABC
% robot.userdata.tracing.trajectory.times: the time of the sample taken
% robot.userdata.tracing.trajectory.start: the beginning index of the trajectory
% robot.userdata.tracing.startTime: time of the first sample (hat)
% robot.userdata.tracing.currentLocation: the current joint coordinates
% robot.userdata.tracing.figure: figure used for the plotting
% 
% MITSUBISHI MELFA TOOLBOX v1.4
% (C) Martin Meloun
% 
% Mitsubishi Melfa Toolbox is free software: you can redistribute it and/or modify
% it under the terms of the GNU Lesser General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% Mitsubishi Melfa Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
% 
% You should have received a copy of the GNU Leser General Public License
% along with Mitsubishi Melfa Toolbox. If not, see <http://www.gnu.org/licenses/>.
% 
% Author:                  Revision:     Date:                    Reason:
% Martin Meloun                    1     18.7.2011 14:54:38       Initial revision

  if (isfield(robot.userdata, 'tracing') && ~isempty(robot.userdata.tracing))
    error('The robot is already recording.');
  end

  robot.userdata.tracing.trajectory.vals = zeros(6, 3000);
  robot.userdata.tracing.trajectory.jCoords = zeros(6, 3000);
  robot.userdata.tracing.trajectory.times = zeros(1, 3000);
  robot.userdata.tracing.trajectory.vals(:,:) = NaN;
  robot.userdata.tracing.trajectory.times(:) = NaN;
  robot.userdata.tracing.trajectory.jCoords(:,:) = NaN;
  robot.userdata.tracing.trajectory.start = -2998; %next - size
  robot.userdata.tracing.trajectory.next = 2;

  robot.userdata.tracing.startTime = hat;
  robot.userdata.tracing.currentLocation = mmGetPos2(robot, 'J');
  robot.userdata.tracing.trajectory.times(1) = hat - robot.userdata.tracing.startTime;
  robot.userdata.tracing.trajectory.jCoords(:,1) = robot.userdata.tracing.currentLocation;
  robot.userdata.tracing.trajectory.vals(:,1) = mmDkt(robot, robot.userdata.tracing.currentLocation);
  
  if (nargin > 4)
    robot.userdata.tracing.external.plotCallback = externalPlotCallback;
  else
    robot.userdata.tracing.external.plotCallback = [];
  end
  
  if (nargin > 3)
    robot.userdata.tracing.external.callback = externalCallback;
  else
    robot.userdata.tracing.external.callback = [];
  end
  
  if (nargin <= 2)
    plotUpdateFrequency = 4;
  end
  

  
  
  if (livePlot)
    robot.userdata.tracing.figure = figure('Name',[robot.description ' [Live]'], 'NumberTitle', 'off');
    jCoords = mmGetPos2(robot, 'J');
    robot.userdata.tracing.plotHandle=newplot;
    robot.userdata.tracing.rtrobot=mmrtCreateInteractiveRobot(robot);
    plot(robot.userdata.tracing.rtrobot, (jCoords')*pi/180);
    robot.userdata.tracing.staticPlots=get(robot.userdata.tracing.plotHandle, 'Children');
    robot.userdata.tracing.plotTimer = timer('TimerFcn', @PlotCallback, 'Period', 1 / plotUpdateFrequency, 'ExecutionMode', 'fixedSpacing','BusyMode','drop','UserData',robot);
    start(robot.userdata.tracing.plotTimer);
  else
    robot.userdata.tracing.plotTimer = [];
  end

end

function PlotCallback(obj, ~)

  robot = get(obj, 'UserData');

  if (robot.userdata.tracing.trajectory.start < 1)
    start = 1;
  else
    start = robot.userdata.tracing.trajectory.start;
  end

  set(0,'CurrentFigure', robot.userdata.tracing.figure);
  jCoor = (robot.userdata.tracing.currentLocation')*pi/180;
  
  %clear everything except the slider and the robot 
  children=get(robot.userdata.tracing.plotHandle, 'Children');
  for j=1:length(children);

    %something not to be redrawn
    if (any(robot.userdata.tracing.staticPlots==children(j)))
      continue;
    end

    delete(children(j));
  end

  mmPlotTrajectory(robot, robot.userdata.tracing.trajectory.vals, 0, start, 0);
  
  %robot toolbox update plot (@#***&%##*)
  r = findobj('Tag', robot.userdata.tracing.rtrobot.name);
  rr = get(r, 'UserData');
  rr.q = jCoor;
  set(r, 'UserData', rr);
  
  %clear the figure
  plot(rr, jCoor);
  
  if (~isempty(robot.userdata.tracing.external.plotCallback))
    robot.userdata.tracing.external.plotCallback(robot);
  end

  drawnow;

end
